@@ -6,14 +6,15 @@
 #' @param min.rc Minimal allowed read count per base (options silenced for VarScan vcf). Bases covered by less than min.rc reads are discarded and considered as sequencing error. For instance, if nucleotides A, C, G and T are covered by respectively 100, 15, 0 and 1 over all the pools, setting min.rc to 0 will lead to discard the position (the polymorphism being considered as tri-allelic), while setting min.rc to 1 (or 2, 3..14) will make the position be considered as a SNP with two alleles A and C (the only read for allele T being disregarded). For VarScan vcf, markers with more than one alternative allele are discarded because the VarScan AD field only contains one alternate read count.
 #' @param min.cov.per.pool Minimal allowed read count (per pool). If at least one pool is not covered by at least min.cov.perpool reads, the position is discarded
 #' @param max.cov.per.pool Maximal allowed read count (per pool). If at least one pool is covered by more than min.cov.perpool reads, the position is discarded
-#' @param min.maf Minimal allowed Minor Allele Frequency (computed from the ratio overal read counts for the reference allele over the read coverage)
+#' @param min.maf Minimal allowed Minor Allele Frequency (computed from the ratio overall read counts for the reference allele over the read coverage)
 #' @param remove.indels Remove indels identified using the number of characters of the alleles in the REF or ALT fields (i.e., if at least one allele is more than 1 character, the position is discarded)
 #' @param nlines.per.readblock Number of Lines read simultaneously. Should be adapted to the available RAM.
+#' @param verbose If TRUE extra information is printed on the terminal
 #' @return A pooldata object containing 7 elements:
 #' \enumerate{
 #' \item "refallele.readcount": a matrix with nsnp rows and npools columns containing read counts for the reference allele (chosen arbitrarily) in each pool
 #' \item "readcoverage": a matrix with nsnp rows and npools columns containing read coverage in each pool
-#' \item "snp.info": a matrix with nsnp rows and four columns containing respectively the contig (or chromosome) name (1st column) and position (2nd column) of the SNP; the allele in the reference assembly (3rd column); the allele taken as reference in the refallele matrix.readcount matrix (4th column); and the alternative allele (5th column)
+#' \item "snp.info": a matrix with nsnp rows and four columns containing respectively the contig (or chromosome) name (1st column) and position (2nd column) of the SNP; the allele taken as reference in the refallele.readcount matrix (3rd column); and the alternative allele (4th column)
 #' \item "poolsizes": a vector of length npools containing the haploid pool sizes
 #' \item "poolnames": a vector of length npools containing the names of the pools
 #' \item "nsnp": a scalar corresponding to the number of SNPs
@@ -24,12 +25,12 @@
 #'  make.example.files(writing.dir=tempdir())
 #'  pooldata=vcf2pooldata(vcf.file=paste0(tempdir(),"/ex.vcf.gz"),poolsizes=rep(50,15))
 #' @export
-vcf2pooldata<-function(vcf.file="",poolsizes=NA,poolnames=NA,min.cov.per.pool=-1,min.rc=1,max.cov.per.pool=1e6,min.maf=0.01,remove.indels=FALSE,nlines.per.readblock=1000000){
+vcf2pooldata<-function(vcf.file="",poolsizes=NA,poolnames=NA,min.cov.per.pool=-1,min.rc=1,max.cov.per.pool=1e6,min.maf=-1,remove.indels=FALSE,nlines.per.readblock=1000000,verbose=TRUE){
   if(nchar(vcf.file)==0){stop("ERROR: Please provide the name of the vcf file as generated by e.g. VarScan")}
   if(sum(is.na(poolsizes))>0){stop("ERROR: Please provide a vector of Pool Sizes (poolsize argument)")}
   ##### Recup info and check validity of argument
   poolsizes=as.numeric(poolsizes)
-  cat("Reading Header lines\n")
+  if(verbose){cat("Reading Header lines\n")}
   file.con=file(vcf.file,open="r") 
   continue.reading=TRUE
   nlines.header=0
@@ -56,7 +57,7 @@ vcf2pooldata<-function(vcf.file="",poolsizes=NA,poolnames=NA,min.cov.per.pool=-1
   continue.reading=TRUE
   time1=proc.time()
   nlines.read=0
-  cat("Parsing allele counts\n")
+  if(verbose){cat("Parsing allele counts\n")}
   while(continue.reading){
     tmp.data=matrix(scan(file=file.con,nlines = nlines.per.readblock,what="character",quiet=TRUE),ncol=npools+9,byrow=T) 
     tmp.nlines.read=nrow(tmp.data)
@@ -71,10 +72,14 @@ vcf2pooldata<-function(vcf.file="",poolsizes=NA,poolnames=NA,min.cov.per.pool=-1
       }
       if(length(rd.index)==0){
         VARSCAN=FALSE
-        cat("Standard format (i.e., as in Bcftools, GATK, etc.) detected for the AD field: the read count for all the alleles identified are separated by a comma\n")
+        if(verbose){
+          cat("Standard format (i.e., as in Bcftools, GATK, etc.) detected for the AD field: the read count for all the alleles identified are separated by a comma\n")
+        }
       }else{
         VARSCAN=TRUE
-        cat("VarScan like format detected for allele count data: the AD field contains allele depth for the alternate allele and RD field for the reference allele (N.B., positions with more than one alternate allele will be ignored)\n")
+        if(verbose){
+        cat("VarScan like format detected for allele count data: the AD field contains allele depth\nfor the alternate allele and RD field for the reference allele\n(N.B., positions with more than one alternate allele will be ignored)\n")
+        }
       }
     }
     #discard monomorphic positions
@@ -97,7 +102,8 @@ vcf2pooldata<-function(vcf.file="",poolsizes=NA,poolnames=NA,min.cov.per.pool=-1
     if(npos>1){
       if(VARSCAN){
         tmp.extract=.extract_vscan_counts(tmp.data[,-1:-9],ad_idx = ad.index,rd_idx = rd.index)
-        tmp.Y=tmp.extract[,1:npools] ; tmp.N=tmp.extract[,(npools+1):(2*npools)]
+        tmp.Y=tmp.extract[,1:npools]
+        tmp.N=tmp.extract[,(npools+1):(2*npools)]
         tmp.maf=0.5-abs(0.5-rowSums(tmp.Y)/rowSums(tmp.N))
         tmp.snpdet=tmp.data[,c(1,2,4,5)]
         rm(tmp.extract)
@@ -125,11 +131,13 @@ vcf2pooldata<-function(vcf.file="",poolsizes=NA,poolnames=NA,min.cov.per.pool=-1
         snpdet=rbind(snpdet,tmp.snpdet)
       }
       nlines.read=nlines.read+tmp.nlines.read
+      if(verbose){
       time.elapsed=(proc.time()-time1)[3]
       nhours=floor(time.elapsed/3600)
       nminutes=floor((time.elapsed-nhours*3600)/60)
       nseconds=round(time.elapsed-nhours*3600-nminutes*60)
       cat(nlines.read," lines processed in",nhours,"h ",nminutes, "m ",nseconds,"s :",nrow(data.Y),"SNPs found\n")
+      }
     }
   }
   close(file.con)
@@ -139,11 +147,15 @@ vcf2pooldata<-function(vcf.file="",poolsizes=NA,poolnames=NA,min.cov.per.pool=-1
   res@nsnp=nrow(data.Y)
   res@refallele.readcount=data.Y ; rm(data.Y)
   res@readcoverage=data.N ; rm(data.N)
-  res@snp.info=snpdet[,c(1:4)] 
+  res@snp.info=data.frame(Chromosome=as.character(snpdet[,1]),
+                          Position=as.numeric(snpdet[,2]),
+                          RefAllele=as.character(snpdet[,3]),
+                          AltAllele=as.character(snpdet[,4]),
+                          stringsAsFactors = FALSE)
   rm(snpdet)
   res@poolsizes=poolsizes
   res@poolnames=poolnames
   
-  cat("Data consists of",res@nsnp,"SNPs for",res@npools,"Pools\n")
+  if(verbose){cat("Data consists of",res@nsnp,"SNPs for",res@npools,"Pools\n")}
   return(res)
 }
\ No newline at end of file
